require 'json/ext'
require 'time'

module PaulWalker
  class RateLimit
    def self.add(namespace, key, current, max)
      store.set(composite_key(namespace, key),
                {
                current: current,
                max: max,
                timestamp: Time.now.utc
                }.to_json)
      set_last_used(namespace, key)
    end

    def self.get(namespace, key)
      value = store.get(composite_key(namespace, key))
      return nil if value.nil?
      value = JSON.parse(value)
      time = Time.parse(value['timestamp'])
      adjusted_current = Time.now.utc.to_i - time.to_i + value['current']
      value['current'] = adjusted_current <= value['current'] ? adjusted_current : value['max']
      value
    end

    def self.composite_key(namespace, key)
      "#{namespace}:#{key}"
    end

    def self.set_last_used(namespace, key)
      store.set(composite_key(namespace, 'last_used'), key)
    end

    def self.last_used(namespace)
      store.get(composite_key(namespace, 'last_used'))
    end

    def self.store
      store = Store.new
      store.provider
    end
  end
end
